(ns rss-thread-watch.config
  "Functions for working with configuration"
  (:require [clojure.java.io :as io]
            [clojure.edn :as edn]
            [clojure.string :as s]
            [rss-thread-watch.utils :as u])
  (:gen-class))

;; Verification TODO: check if all required keys are included so we don't get nils

(def VERSION "0.4.7")

(def GLOBAL-CONFIG
  "Global config with defaults for missing entires"
  ;; I know globals are ew in Clojure but I don't know any
  ;; better way of doing this
  (atom nil))

;; Internal default config
(def CONFIG-DEFAULT
  "Internal default config"
  {:port 6969
   :default-board "/mlp/"
   :enable-board-listing true
   :served-filename "/feed"
   :board-disabled-message "This board is not enabled for feed generation.\n\nYou can contact me here: [contact] and I may enable it for you"
   :boards-defaults {:refresh-rate 300
                     :starting-page 7
                     :default-chod 94
                     :target "https://api.4chan.org/{board}/catalog.json"
                     :host "https://boards.4chan.org/{board}/thread/{threadnum}"
                     :lazy-load true}
   :boards-enabled {"/mlp/" {:lazy-load false}
                    "/g/" {:starting-page 7}
                    "/po/" {:starting-page 8
                            :refresh-rate 86400}
                    "/p/" {:starting-page 8
                           :refresh-rate 1800}}})

(defn load-config
  "Attempts to load config from file [f].
   Returns loaded config map or nil if failed"
  [f]
  (let [fl (io/as-file f)]
    (when (.exists fl)
      (with-open [r (io/reader fl)]
        (edn/read (java.io.PushbackReader. r))))))


(defn config-url-expand
  "Expands substitution in :target and :host fields"
  [filled-config]
  (let [boards (get filled-config :boards-enabled)
        selecting '(:target :host)
        pattern "{board}"]
    (assoc filled-config
           :boards-enabled
           (u/fmap (fn [board confs]
              (->> (select-keys confs selecting)
                   (u/fmap (fn [k v]
                             (s/replace v pattern (s/replace board "/" ""))))
                   (merge confs)))
            boards))))

(defn config-fill-board-defaults
  "Fills every enabled board with default config values"
  [config]
  (let [defaults (:boards-defaults config)]
    (as-> config conf
      (update-in conf
                 '(:boards-enabled)
                 (fn [mp]
                   (u/fmap (fn [k v]
                             (assoc (u/map-apply-defaults v defaults) :name k))
                           mp)))
      (dissoc conf :boards-defaults)
      (config-url-expand conf))))

(defn get-some-config
  "Attempts to get config somehow,
   first from [custom-file], if it's nil,
   then from ./config.edn file.
   If is neither exists, default internal one is used."
  [custom-file]
  (config-fill-board-defaults
   ;; TODO: There has to be try/catch for when file is invalid edn
   ;; This is gonna be done when config validation comes in Beta 2
   (let [file-to-try (u/nil?-else custom-file
                                  "./config.edn")]
     (u/when-else (load-config file-to-try)
      CONFIG-DEFAULT))))
